/*  file yasep/core-js/group_ie.js
This file is distributed under terms of the Affero GPL v3 license or later, see http://yasep.org
created 2005-06-11 by whygee@f-cpu.org
2007-04-11 : moved from vsp_eu.js to group_ie.js
2007-04-15 : added the USE_ flags
2007-11-05 : rebranding to YASEP, added support for IMM16_8LSB
2008-11-17 : added OPTIONAL|YASEP32_ONLY|YASEP16_ONLY
2008-11-30 : 16-bit action
2009-01-06 : moved things again to match 16-bit VHDL
    order : MOV SB LZB LSB (16-bits + 32-bits)
            SHH SH LZH LSH (32-bits only)
2009-07-19 : added insert/extract aliases
2009-07-28 : added Y
2009-07-29 : FORM_iR
2009-08-07 : s.snd...
2009-08-09 : IGNORE_SND, IGNORE_SI4
2009-08-10 : Y.NO_WRITEBACK | Y.NO_CONDITION | Y.IGNORE_IMM16
2009-08-23 : removed FORM_X
2011-09-17 : added IMM20
2012-09-22 : removing the load/store and keeping the ex-aliases insert/extract
2012-09-25 : MOVH and cleanup
2012-11-23 : MOVH again

This file implements the "IE" (Insert/Extract) Execution Unit
  that does "Move", "load" and "store"

dependency : group_eu.js and all its parents
*/

Y.prefix=Y.GROUP_IE;

// 16/32-bit instructions :

Y.NewOpcode("MOV",
 function(s) { s.result = s.si4 & F8; },
 function(s) { s.result = s.si4 & F4; },
 Y.FORM_RR | Y.FORM_IR | Y.FORM_iR,
 Y.IGNORE_SND | Y.SWAP_IR |  Y.IMM20,
 "Write imm16/imm20/si4 to dest");


Y.NewOpcode("IB",
 function(s) {
   var a=(s.si4 & 3) * 8;
   s.result = ((s.snd & F2) << a) | (s.dst3 & ~(F2 << a));
 },
 function(s) {
   var a=(s.si4 & 1) * 8;
   s.result = ((s.snd & F2) << a) | (s.dst3 & ~(F2 << a));
 },
 Y.FORM_iRR | Y.FORM_RRR,
 Y.READ_DST3 | Y.IMM_LSB,
"Insert a byte into a (half)word");


Y.NewOpcode("EZB",
 function(s) {
   s.result = F2 & (s.snd >> ((s.si4 & 3) * 8));
 },
 function(s) {
   s.result = F2 & (s.snd >> ((s.si4 & 1) * 8));
 },
 Y.FORM_iRR | Y.FORM_RRR,
 Y.IMM_LSB,
 "Extract and Zero-extend Byte");


Y.NewOpcode("ESB",
 function(s) {
   s.result = F2 & (s.snd >> ((s.si4 & 3) * 8));
   if (s.result & 0x80)
       s.result |= F8-F2; // 0xFFFFFF00;
 },
 function(s) {
   s.result = F2 & (s.snd >> ((s.si4 & 1) * 8));
   if (s.result & 0x80)
       s.result |= F4-F2; // 0xFF00;
 },
 Y.FORM_iRR | Y.FORM_RRR,
 Y.IMM_LSB,
 "Extract and Sign-extend Byte");

/////////////////////////////////////////////////////////
// 32-bit instructions :


Y.NewOpcode("MOVH",
 function(s) {
   s.result = (s.snd & F4) | ((s.si4 & F4)<<16);
 },
 null, // YASEP32_ONLY
 Y.FORM_RR | Y.FORM_RRR | Y.FORM_IR | Y.FORM_iR | Y.FORM_IRR | Y.FORM_iRR,
 Y.YASEP32_ONLY | Y.ALIAS_IRR | Y.ALIAS_RR,
  "Shift operand (R/i4/i16) left by 16 bits");


Y.NewOpcode("IH",
 function(s) {
   var a=(s.si4 & 3) * 8;
   s.result = ((s.snd & F4) << a) | (s.dst3 & ~(F4 << a));

   s.carry=0;
   if ((s.si4 & 3) > 2){
     s.carry=1;
     s.result &= F8;
   }
 },
 null, // YASEP32_ONLY
 Y.FORM_iRR | Y.FORM_RRR,
 Y.YASEP32_ONLY |  Y.READ_DST3 | Y.IMM_LSB | Y.CHANGE_CARRY,
"Insert the second operand's Half-word into destination");


Y.NewOpcode("EZH",
 function(s) {
   s.carry=((s.si4 & 3) > 2)?1:0;
   s.result = F4 & (s.snd >>> ((s.si4 & 3) * 8)); // notice the >>> that leaves the MSB cleared in case (s.si4 & 3) > 2
 },
 null, // YASEP32_ONLY
 Y.FORM_iRR | Y.FORM_RRR,
 Y.YASEP32_ONLY | Y.IMM_LSB | Y.CHANGE_CARRY,
 "Extract and Zero-extend Half-word");


Y.NewOpcode("ESH",
 function(s) {
   s.carry=((s.si4 & 3) > 2)?1:0;
   s.result = F4 & (s.snd >>> ((s.si4 & 3) * 8));
   if (s.result & 0x80)
     s.result |= F8-F4; // 0xFFFF0000;
 },
 null, // YASEP32_ONLY
 Y.FORM_iRR | Y.FORM_RRR,
 Y.YASEP32_ONLY | Y.IMM_LSB | Y.CHANGE_CARRY,
 "Extract and Sign-extend Half-word");
